@@ -8,9 +8,15 @@ describe VagrantCloud::Box::Provider do
   let(:box_name) { double("box_name") }
   let(:version_version) { double("version_version") }
   let(:box_url) { double("box_url") }
+  let(:architecture) { "BOX_ARCHITECTURE" }
 
-
-  let(:subject) { described_class.new(version: version, name: provider_name) }
+  let(:subject) {
+    described_class.new(
+      version: version,
+      name: provider_name,
+      architecture: architecture
+    )
+  }
 
   before do
     allow(version).to receive(:is_a?).with(VagrantCloud::Box::Version).and_return(true)
@@ -79,6 +85,12 @@ describe VagrantCloud::Box::Provider do
         subject.delete
       end
 
+      it "should send architecture" do
+        expect(version).to receive_message_chain(:box, :organization, :account, :client, :box_version_provider_delete).
+          with(hash_including(architecture: architecture))
+        subject.delete
+      end
+
       it "should return nil" do
         expect(version).to receive_message_chain(:box, :organization, :account, :client, :box_version_provider_delete)
         expect(subject.delete).to be_nil
@@ -423,12 +435,35 @@ describe VagrantCloud::Box::Provider do
         subject.send(:save_provider)
       end
 
+      it "should include architecture" do
+        expect(version).to receive_message_chain(:box, :organization, :account, :client, :box_version_provider_update).
+          with(hash_including(architecture: architecture)).and_return({})
+        subject.send(:save_provider)
+      end
+
+      it "should include new architecture" do
+        expect(version).to receive_message_chain(:box, :organization, :account, :client, :box_version_provider_update).
+          with(hash_including(new_architecture: architecture)).and_return({})
+        subject.send(:save_provider)
+      end
+
       it "should include URL" do
         subject.url = box_url
         expect(version).to receive_message_chain(:box, :organization, :account, :client, :box_version_provider_update).
           with(hash_including(url: box_url)).and_return({})
         subject.send(:save_provider)
       end
+
+      context "when architecture is changed" do
+        let(:new_architecture) { "NEW_BOX_ARCHITECTURE" }
+
+        it "should include original and new architectures" do
+          subject.architecture = new_architecture
+          expect(version).to receive_message_chain(:box, :organization, :account, :client, :box_version_provider_update).
+            with(hash_including(new_architecture: new_architecture, architecture: architecture)).and_return({})
+          subject.send(:save_provider)
+        end
+      end
     end
 
     context "when provider does not exist" do
@@ -482,6 +517,12 @@ describe VagrantCloud::Box::Provider do
         subject.send(:save_provider)
       end
 
+      it "should include architecture" do
+        expect(version).to receive_message_chain(:box, :organization, :account, :client, :box_version_provider_create).
+          with(hash_including(architecture: architecture)).and_return({})
+        subject.send(:save_provider)
+      end
+
       it "should include URL" do
         subject.url = box_url
         expect(version).to receive_message_chain(:box, :organization, :account, :client, :box_version_provider_create).
